using System.Reflection;
using System.Text.RegularExpressions;
using FitConnect.Models;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json.Linq;
using Newtonsoft.Json.Schema;
using JsonSchema = NJsonSchema.JsonSchema;

namespace FitConnect.Encryption;

public static class JsonHelper {
    public static bool ValidateJsonSchema(string schema, string json, ILogger? logger = null) {
        var jSchema = JSchema.Parse(schema);
        return JObject.Parse(json).IsValid(jSchema);
    }

    /// <summary>
    ///     Verify the metadata hash and content to fit the schema
    /// </summary>
    /// <param name="metadataString"></param>
    /// <param name="schema"></param>
    /// <param name="logger">Optional logger, to print errors to logger</param>
    /// <returns></returns>
    public static async Task<VerificationResult> VerifyMetadata(string metadataString,
        JsonSchema? schema,
        ILogger? logger = null) {
        schema ??= await LoadJsonSchemaFromResource("metadata.schema.json");

        var schemaName = schema.ExtensionData["$id"].ToString();


        if (!(schemaName == null || new Regex(Metadata.SchemaPattern).IsMatch(schemaName))) {
            logger?.LogError("Unsupported metadata schema version");
            return false;
        }

        var result = schema.Validate(metadataString).ToList();
        if (result.Count > 0)
            logger?.LogError("Validation failed with {Errors}",
                result.Aggregate("", (a, b) => a + "\n\t" + b));
        return new VerificationResult(result.Count == 0, result);
    }


    internal static string LoadContentOfResource(string resourceName) {
        var assembly = Assembly.GetExecutingAssembly();
        var fullQualifiedName = $"{assembly.GetName().Name}.{resourceName}";
        var resourceStream = assembly.GetManifestResourceStream(fullQualifiedName);
        if (resourceStream == null)
            throw new Exception($"Resource {fullQualifiedName} not found");
        var reader = new StreamReader(resourceStream);
        
        // Newtonsoft has currently a problem, that it does not support $defs in JSON schemas
        // https://github.com/RicoSuter/NJsonSchema/issues/1536
        return reader.ReadToEnd().Replace("$defs", "definitions");
    }

    internal static Task<JsonSchema> LoadJsonSchemaFromResource(string schemaName)
        => JsonSchema.FromJsonAsync(LoadContentOfResource($"JsonSchemas.{schemaName}"));

    public static async Task<bool> ValidateJsonSchema(Uri schema, string jsonData) {
        var schemaResponse = await new HttpClient()
            .GetAsync(schema);

        return ValidateJsonSchema(await schemaResponse.Content.ReadAsStringAsync(), jsonData);
    }
}
