using System.Runtime.CompilerServices;
using FitConnect.Validation;
using FluentAssertions;
using Microsoft.Extensions.Logging;
using Moq;

namespace ValidationTests;

public class SubmissionSchemaValidationTests {
    private readonly SchemaValidationVisitor _schemaValidationVisitor;
    private readonly Moq.Mock<ILogger<SchemaValidationVisitor>> _logger = new();

    public SubmissionSchemaValidationTests() {
        _schemaValidationVisitor = new SchemaValidationVisitor(logger: _logger.Object);
    }

    [Fact]
    public async Task SimpleSchemaTest_CommonTest_ShouldPass() {
        var jsonData = """
        {
            "name": "test",
            "age": 10
        }
        """;

        var jsonSchema = """
        {
            "type": "object",
            "properties": {
                "name": {
                    "type": "string"
                },
                "age": {
                    "type": "number"
                }
            }
        }
        """;

        var validationResult =
            await _schemaValidationVisitor.ValidateSubmission(jsonData, jsonSchema);
        validationResult.Should().BeTrue();
    }

    [Fact]
    public async Task SimpleSchemaTest_InvalidJson_ShouldFail() {
        var jsonData = """
        {
            "name": "test",
            "age": "Wrong Value"
        }
        """;

        var jsonSchema = """
        {
            "type": "object",
            "properties": {
                "name": {
                    "type": "string"
                },
                "age": {
                    "type": "number"
                }
            }
        }
        """;

        var validationResult =
            await _schemaValidationVisitor.ValidateSubmission(jsonData, jsonSchema);
        validationResult.Should().BeFalse();
    }

    [Fact]
    public async Task FileSchemaTest_SchemaFromFile_ShouldPass() {
        var jsonData = """
        {
            "firstName": "John",
            "lastName": "Doe",
            "age": 21
        }
        """;

        var jsonSchema = "./simpleSchema.json";

        var validationResult =
            await _schemaValidationVisitor.ValidateSubmission(jsonData, jsonSchema);
        validationResult.Should().BeTrue();
    }
    
    [Fact]
    public async Task FileSchemaTest_SchemaFromFileMissingFirstName_ShouldFail() {
        var jsonData = """
        {
            "lastName": "Doe",
            "age": 21
        }
        """;

        var jsonSchema = "./simpleSchema.json";

        var validationResult =
            await _schemaValidationVisitor.ValidateSubmission(jsonData, jsonSchema);
        validationResult.Should().BeFalse();
    }
    [Fact]
    public async Task FileSchemaTest_SchemaFromUrl_ShouldPass() {
        var jsonData = """
        {
            "name": "John Doe",
            "email": "john.doe@example.com",
            "age": 21
        }
        """;

        var jsonSchema = "https://git.fitko.de/fit-connect/sdk-dotnet/-/raw/main/Examples/example.schema.json";

        var validationResult =
            await _schemaValidationVisitor.ValidateSubmission(jsonData, jsonSchema);
        validationResult.Should().BeTrue();
    }
    
    [Fact]
    public async Task FileSchemaTest_SchemaFromUrlMissingName_ShouldFail() {
        var jsonData = """
        {
            "email": "john.doe@example.com",
            "age": 21
        }
        """;

        var jsonSchema = "https://git.fitko.de/fit-connect/sdk-dotnet/-/raw/main/Examples/example.schema.json";

        var validationResult =
            await _schemaValidationVisitor.ValidateSubmission(jsonData, jsonSchema);
        validationResult.Should().BeFalse();
    }
}
